/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package main;

import static opengl.GL.*;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.lwjgl.LWJGLException;
import org.lwjgl.input.Keyboard;
import org.lwjgl.opengl.Display;
import org.lwjgl.util.vector.Matrix4f;
import org.lwjgl.util.vector.Vector2f;
import org.lwjgl.util.vector.Vector3f;
import util.Util;

/**
 *
 * @author Sascha Kolodzey, Nico Marniok
 */
public class CubeAndPyramid {
    private static int programID;       // shader program id
    private static int quadVAID;        // vertexarrayobject id of the cube
    private static int triangleVAID;    // vertexarrayobject id of the pyramid
    private static int modelLocation;   // uniform location of "model"
    private static final Matrix4f cubeModel[] = new Matrix4f[] {
        new Matrix4f(), new Matrix4f(), new Matrix4f(), new Matrix4f(),
    };
    private static final Matrix4f pyramidModel[] = new Matrix4f[] {
        new Matrix4f(), new Matrix4f(), new Matrix4f(), new Matrix4f(),
    };
    
    private static final Vector3f camPos = new Vector3f(0.0f, 0.0f, -4.0f);
    private static final Vector2f moveDir = new Vector2f(0.0f, 0.0f);
    private static boolean bContinue = true;
    
    public static void main(String[] argv) {
        try {
            init();
            programID = Util.createShaderProgram("shader/Main_VS.glsl", "shader/VertexColor_FS.glsl");
            modelLocation = glGetUniformLocation(programID, "model");
            quadVAID = Util.createQuad();
            triangleVAID = Util.createTriangle();
            glPointSize(16.0f);
            glLineWidth(4.0f);
            initCube();
            initPyramid();
            render();
            Display.destroy();
        } catch (LWJGLException ex) {
            Logger.getLogger(CubeAndPyramid.class.getName()).log(Level.SEVERE, null, ex);
        }
    }
    
    public static void render() throws LWJGLException {
        glClearColor(0.0f, 0.0f, 0.25f, 1.0f); // dark blue
        long last = System.currentTimeMillis();
        long now;
        while(bContinue && !Display.isCloseRequested()) {            
            glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
            glUseProgram(programID);
            
            for(Matrix4f cubeSide : cubeModel) {
                cubeSide.store(Util.MAT_BUFFER);
                Util.MAT_BUFFER.position(0);
                glUniformMatrix4(modelLocation, false, Util.MAT_BUFFER);
                glBindVertexArray(quadVAID);
                glDrawArrays(GL_LINE_LOOP, 0, 4);
            }
            
            for(Matrix4f pyramidSide : pyramidModel) {
                pyramidSide.store(Util.MAT_BUFFER);
                Util.MAT_BUFFER.position(0);
                glUniformMatrix4(modelLocation, false, Util.MAT_BUFFER);
                glBindVertexArray(triangleVAID);
                glDrawArrays(GL_LINE_LOOP, 0, 3);
            }
            
            Display.update();
            Display.sync(60);
            
            now = System.currentTimeMillis();
            handleInput(now - last);
            updateUniforms();
            last = now;
        }
    }
    
    public static void initCube() {
        // TODO: Aufgabe 3.1
    }
    
    public static void initPyramid() {
        // TODO: Aufgabe 3.2
    }
    
    public static void handleInput(long millis) {
        float moveSpeed = 2e-3f*(Keyboard.isKeyDown(Keyboard.KEY_LSHIFT) ? 2.0f : 1.0f)*(float)millis;
        
        while(Keyboard.next()) {
            if(Keyboard.getEventKeyState()) {
                switch(Keyboard.getEventKey()) {
                    case Keyboard.KEY_W: moveDir.y += 1.0f; break;
                    case Keyboard.KEY_S: moveDir.y -= 1.0f; break;
                    case Keyboard.KEY_A: moveDir.x += 1.0f; break;
                    case Keyboard.KEY_D: moveDir.x -= 1.0f; break;
                }
            } else {
                switch(Keyboard.getEventKey()) {
                    case Keyboard.KEY_W: moveDir.y -= 1.0f; break;
                    case Keyboard.KEY_S: moveDir.y += 1.0f; break;
                    case Keyboard.KEY_A: moveDir.x -= 1.0f; break;
                    case Keyboard.KEY_D: moveDir.x += 1.0f; break;
                }
            }
        }
        
        camPos.x += moveSpeed * moveDir.x;
        camPos.y += moveSpeed * moveDir.y;
        camPos.z = 1.0f;
        
        Util.clamp(camPos, -1.0f, +1.0f, camPos);
        
        if(Keyboard.isKeyDown(Keyboard.KEY_ESCAPE)) bContinue = false;
    }
    
    public static void updateUniforms() {        
        Matrix4f view = Util.lookAtRH(camPos, new Vector3f(), new Vector3f(0.0f, 1.0f, 0.0f), null);
        Matrix4f projection = new Matrix4f();
        Util.orthographicRH(projection);
        Matrix4f viewProj = Matrix4f.mul(projection, view, null);
        viewProj.store(Util.MAT_BUFFER);
        Util.MAT_BUFFER.position(0);
        int viewProjLoc = glGetUniformLocation(programID, "viewProj");
        glUniformMatrix4(viewProjLoc, false, Util.MAT_BUFFER);
    }
}
